#include "Polinomio.hpp"

Polinomio :: Polinomio ( double coeff = 0.0 , unsigned esp = 0 ) {
    elem = 0;
    pol = NULL;
    if( 0.0 != coeff ) {
        elem = esp+1;
        pol = new double[elem];
        for(unsigned i = 0; i < esp+1; i++)
            pol[i] = 0.0;
        pol[esp] = coeff;
    }
}

Polinomio :: Polinomio ( const Polinomio & p ) {
    elem = p.elem;
    pol = new double[p.elem];
    for( unsigned i = 0; i < elem; i++ ) {
        pol[i] = p.pol[i];   
    }
}

Polinomio :: ~Polinomio () {
    elem = 0;
    delete [] pol;
    pol = NULL;
}

double Polinomio :: Coefficiente ( unsigned u ) {
    if( u < elem && u >= 0 )
        return pol[u];
    else
        return 0;
}

void Polinomio :: operator= ( const Polinomio & p ) {
    if ( NULL != pol )
        delete [] pol;
    elem = p.elem;
    pol = new double[p.elem];
    for( unsigned i = 0; i < elem; i++ ) {
        pol[i] = p.pol[i];   
    }
}

Polinomio operator+ ( const Polinomio & p1 , const Polinomio & p2 ) {
    Polinomio p;
    //Genero il vettore del polinomio grande come il pi grande
    //tra i due polinomi
    if( p1.elem >= p2.elem )
        p.elem = p1.elem;
    else
        p.elem = p2.elem;
    p.pol = new double[p.elem];
    //Azzero il polinomio
    for( unsigned i = 0; i < p.elem; i++ )
        p.pol[i] = 0.0;
    //Copio i due polinomi
    for( unsigned i = 0; i < p1.elem; i++ )
        p.pol[i] += p1.pol[i];
    for( unsigned i = 0; i < p2.elem; i++ )
        p.pol[i] += p2.pol[i];
    return p;
}

Polinomio operator- ( const Polinomio & p1 , const Polinomio & p2 ) {
    Polinomio p;
    //Genero il vettore del polinomio grande come il pi grande
    //tra i due polinomi
    if( p1.elem >= p2.elem )
        p.elem = p1.elem;
    else
        p.elem = p2.elem;
    p.pol = new double[p.elem];
    //Azzero il polinomio
    for( unsigned i = 0; i < p.elem; i++ )
        p.pol[i] = 0.0;
    //Copio i due polinomi
    for( unsigned i = 0; i < p1.elem; i++ )
        p.pol[i] += p1.pol[i];
    for( unsigned i = 0; i < p2.elem; i++ )
        p.pol[i] -= p2.pol[i];
    return p;
}
        
ostream & operator<< ( ostream & os , const Polinomio & p ) {
    os << "<";
    for( int i = p.elem-1; i >= 0; i-- ) {
        if( 0.0 != p.pol[i] ) {
            if( p.pol[i] > 0 && i != p.elem-1 )
                os << " +";
            else
                os << " ";
            os << p.pol[i] << "x^" << i;
        }
    }
    os << ">";
    return os;
}

istream & operator>> ( istream & is , Polinomio & p ) {
    char ch;
    double d;
    unsigned esp;
    is >> ch;           //'<'
    //Acquisisco il primo valore, vedo l'esponente e genero
    //direttamente il vettore adatto
    ch = is.peek();
    if( ch == '>' ) {
        p.elem = 0;
        p.pol = NULL;
        return is;
    }
    //Il polinomio non  nullo
    is >> d;
    is >> ch >> ch;     //'x' '^'
    is >> esp;
    //Trovato il grado massimo, genero il vettore e carico l'elemento
    p.elem = esp+1;
    p.pol = new double[p.elem];
    for(unsigned i = 0; i < p.elem; i++)
        p.pol[i] = 0;
    p.pol[esp] = d;
    do {
        ch = is.peek();
        if(ch != '>') {
            is >> ch;       //'+'
            is >> d;
            is >> ch >> ch; //'x' '^'
            is >> esp;
            p.pol[esp] = d;
        }
    } while ( ch == '>' );
    return is;
}
