#include "complessi.hpp"

#include <cmath>
using namespace std;

Complesso :: Complesso ( double r = 0.0, double i = 0.0 )
{
    re = r;
    im = i;
}

double Complesso :: Modulo ()
{
    return sqrt(re*re+im*im);
}

double Complesso :: Fase ()
{
    if( im >= 0)
        return acos(re/Modulo());
    else
        return -acos(re/Modulo());
}

Complesso operator+ ( Complesso &c1 , Complesso &c2 ) {
    Complesso c;
    c.re = c1.re + c2.re;
    c.im = c1.im + c2.im;
    return c;
}

Complesso operator- ( Complesso &c1 , Complesso &c2 ) {
    Complesso c;
    c.re = c1.re - c2.re;
    c.im = c1.im - c2.im;
    return c;
}

Complesso operator* ( Complesso &c1 , Complesso &c2 ) {
    Complesso c;
    //(Re1+iIm1)(Re2+iIm2) = Re1*Re2 + Re1*iIm2 + iIm1*Re2 + iIm1*iIm2
    //                     = (Re1*Re2 - Im1*Im2) + i(Re1*Im2 + Re2*Im1)
    c.re = c1.re * c2.re - c1.im * c2.im;
    c.im = c1.re * c2.im + c2.re * c1.im;
    return c;
}

Complesso operator/ ( Complesso &c1 , Complesso &c2 ) {
    Complesso c;
    // (Re1+iIm1)   (Re1+iIm1)   (Re2-iIm2)   (Re1+iIm1)(Re2-iIm2)   (Re1*Re2+Im1*Im2)    (Re2*Im1-Re1*Im2)
    // ---------- = ---------- * ---------- = -------------------- = ----------------- + i-----------------
    // (Re2+iIm2)   (Re2+iIm2)   (Re2-iIm2)      (Re2^2+Im2^2)         (Re2^2+Im2^2)        (Re2^2+Im2^2)
    c.re = (c1.re * c2.re + c1.im * c2.re)/(c2.re * c2.re + c2.im * c2.im);
    c.im = (c2.re * c1.im - c1.re * c2.im)/(c2.re * c2.re + c2.im * c2.im);
    return c;
}
